class HtmlUtils {

  /// 转义 html 字符
  static String contentSymbolToEscape(String text) {
    if (text.isEmpty) {
      return text;
    }
    String result = '';
    for (var i = 0; i < text.length; i++) {
      var ch = text[i];
      final s = symbols.entries.firstWhere((element) => element.key == ch, orElse: () => const MapEntry<String, String>("", ""));
      if (s.key.isNotEmpty) {
        result += s.value;
      } else {
        result += ch;
      }
    }
    return result;
  }

  /// 还原 html 字符
  static String contentEscapeBackToSymbol(String text) {
    if (text.isEmpty) {
      return text;
    }
    String result = text;
    symbols.forEach((key, value) {
      if (result.contains(value)) {
        result = result.replaceAll(value, key);
      }
    });
    return result;
  }



  static final symbols = {
    '"': '&quot;',
    '\'': '&#039;',
    '>': '&gt;',
    '¡': '&iexcl;',
    '£': '&pound;',
    '¥': '&yen;',
    '§': '&sect;',
    '©': '&copy;',
    '«': '&laquo;',
    '®': '&reg;',
    '¯': '&macr;',
    '±': '&plusmn;',
    '³': '&sup3;',
    'µ': '&micro;',
    '·': '&middot;',
    '¹': '&sup1;',
    '»': '&raquo;',
    '½': '&frac12;',
    '¿': '&iquest;',
    'Á': '&Aacute;',
    'Ã': '&Atilde;',
    'Å': '&Aring;',
    'Ç': '&Ccedil;',
    'É': '&Eacute;',
    'Ë': '&Euml;',
    'Í': '&Iacute;',
    'Ï': '&Iuml;',
    'Ñ': '&Ntilde;',
    'Ó': '&Oacute;',
    'Õ': '&Otilde;',
    '×': '&times;',
    'Ù': '&Ugrave;',
    'Û': '&Ucirc;',
    'Ý': '&Yacute;',
    'ß': '&szlig;',
    'á': '&aacute;',
    'ã': '&atilde;',
    'å': '&aring;',
    'ç': '&ccedil;',
    'é': '&eacute;',
    'ë': '&euml;',
    'í': '&iacute;',
    'ï': '&iuml;',
    'ñ': '&ntilde;',
    'ó': '&oacute;',
    'õ': '&otilde;',
    '÷': '&divide;',
    'ù': '&ugrave;',
    'û': '&ucirc;',
    'ý': '&yacute;',
    'ÿ': '&yuml;',
    'œ': '&oelig;',
    'š': '&scaron;',
    'ƒ': '&fnof;',
    '˜': '&tilde;',
    'Β': '&Beta;',
    'Δ': '&Delta;',
    'Ζ': '&Zeta;',
    'Θ': '&Theta;',
    'Κ': '&Kappa;',
    'Μ': '&Mu;',
    'Ξ': '&Xi;',
    'Π': '&Pi;',
    'Σ': '&Sigma;',
    'Υ': '&Upsilon;',
    'Χ': '&Chi;',
    'Ω': '&Omega;',
    'β': '&beta;',
    'δ': '&delta;',
    'ζ': '&zeta;',
    'θ': '&theta;',
    'κ': '&kappa;',
    'μ': '&mu;',
    'ξ': '&xi;',
    'π': '&pi;',
    'ς': '&sigmaf;',
    'τ': '&tau;',
    'φ': '&phi;',
    'ψ': '&psi;',
    'ϑ': '&thetasym;',
    'ϖ': '&piv;',
    '–': '&ndash;',
    '‘': '&lsquo;',
    '‚': '&sbquo;',
    '”': '&rdquo;',
    '†': '&dagger;',
    '•': '&bull;',
    '‰': '&permil;',
    '″': '&Prime;',
    '›': '&rsaquo;',
    '⁄': '&frasl;',
    'ℑ': '&image;',
    'ℜ': '&real;',
    'ℵ': '&alefsym;',
    '↑': '&uarr;',
    '↓': '&darr;',
    '↵': '&crarr;',
    '⇑': '&uArr;',
    '⇓': '&dArr;',
    '∀': '&forall;',
    '∃': '&exist;',
    '∇': '&nabla;',
    '∉': '&notin;',
    '∏': '&prod;',
    '−': '&minus;',
    '√': '&radic;',
    '∞': '&infin;',
    '∧': '&and;',
    '∩': '&cap;',
    '∫': '&int;',
    '∼': '&sim;',
    '≈': '&asymp;',
    '≡': '&equiv;',
    '≥': '&ge;',
    '⊃': '&sup;',
    '⊆': '&sube;',
    '⊕': '&oplus;',
    '⊥': '&perp;',
    '⌈': '&lceil;',
    '⌊': '&lfloor;',
    '⟨': '&lang;',
    '◊': '&loz;',
    '♣': '&clubs;',
    '♦': '&diams;',
    '&': '&amp;',
    '<': '&lt;',
    ' ': '&nbsp;',
    '¢': '&cent;',
    '¤': '&curren;',
    '¦': '&brvbar;',
    '¨': '&uml;',
    'ª': '&ordf;',
    '¬': '&not;',
    '°': '&deg;',
    '²': '&sup2;',
    '´': '&acute;',
    '¶': '&para;',
    '¸': '&cedil;',
    'º': '&ordm;',
    '¼': '&frac14;',
    '¾': '&frac34;',
    'À': '&Agrave;',
    'Â': '&Acirc;',
    'Ä': '&Auml;',
    'Æ': '&AElig;',
    'È': '&Egrave;',
    'Ê': '&Ecirc;',
    'Ì': '&Igrave;',
    'Î': '&Icirc;',
    'Ð': '&ETH;',
    'Ò': '&Ograve;',
    'Ô': '&Ocirc;',
    'Ö': '&Ouml;',
    'Ø': '&Oslash;',
    'Ú': '&Uacute;',
    'Ü': '&Uuml;',
    'Þ': '&THORN;',
    'à': '&agrave;',
    'â': '&acirc;',
    'ä': '&auml;',
    'æ': '&aelig;',
    'è': '&egrave;',
    'ê': '&ecirc;',
    'ì': '&igrave;',
    'î': '&icirc;',
    'ð': '&eth;',
    'ò': '&ograve;',
    'ô': '&ocirc;',
    'ö': '&ouml;',
    'ø': '&oslash;',
    'ú': '&uacute;',
    'ü': '&uuml;',
    'þ': '&thorn;',
    'Œ': '&OElig;',
    'Š': '&Scaron;',
    'Ÿ': '&Yuml;',
    'ˆ': '&circ;',
    'Α': '&Alpha;',
    'Γ': '&Gamma;',
    'Ε': '&Epsilon;',
    'Η': '&Eta;',
    'Ι': '&Iota;',
    'Λ': '&Lambda;',
    'Ν': '&Nu;',
    'Ο': '&Omicron;',
    'Ρ': '&Rho;',
    'Τ': '&Tau;',
    'Φ': '&Phi;',
    'Ψ': '&Psi;',
    'α': '&alpha;',
    'γ': '&gamma;',
    'ε': '&epsilon;',
    'η': '&eta;',
    'ι': '&iota;',
    'λ': '&lambda;',
    'ν': '&nu;',
    'ο': '&omicron;',
    'ρ': '&rho;',
    'σ': '&sigma;',
    'υ': '&upsilon;',
    'χ': '&chi;',
    'ω': '&omega;',
    'ϒ': '&upsih;',
    '—': '&mdash;',
    '’': '&rsquo;',
    '“': '&ldquo;',
    '„': '&bdquo;',
    '‡': '&Dagger;',
    '…': '&hellip;',
    '′': '&prime;',
    '‹': '&lsaquo;',
    '‾': '&oline;',
    '€': '&euro;',
    '℘': '&weierp;',
    '™': '&trade;',
    '←': '&larr;',
    '→': '&rarr;',
    '↔': '&harr;',
    '⇐': '&lArr;',
    '⇒': '&rArr;',
    '⇔': '&hArr;',
    '∂': '&part;',
    '∅': '&empty;',
    '∈': '&isin;',
    '∋': '&ni;',
    '∑': '&sum;',
    '∗': '&lowast;',
    '∝': '&prop;',
    '∠': '&ang;',
    '∨': '&or;',
    '∪': '&cup;',
    '∴': '&there4;',
    '≅': '&cong;',
    '≠': '&ne;',
    '≤': '&le;',
    '⊂': '&sub;',
    '⊄': '&nsub;',
    '⊇': '&supe;',
    '⊗': '&otimes;',
    '⋅': '&sdot;',
    '⌉': '&rceil;',
    '⌋': '&rfloor;',
    '⟩': '&rang;',
    '♠': '&spades;',
    '♥': '&hearts;'
  };
}
